sysfonts::font_add("Invention", regular = "Invention_Lt.ttf")

hexSticker::sticker(
  subplot = ~ plot.new(),
  s_x = 1,
  s_y = 1,
  s_width = 0.1,
  s_height = 0.1,
  package = "py-pkglite",
  p_x = 1.02,
  p_y = 1.05,
  p_size = 26,
  h_size = 1.2,
  p_family = "Invention",
  p_color = "#FFF",
  h_fill = "#00857C",
  h_color = "#005c55",
  dpi = 320,
  filename = "docs/assets/logo.png"
)
